@@ -97,6 +97,7 @@ TEST_DECLARE   (unref_in_prepare_cb)
 TEST_DECLARE   (timer_ref)
 TEST_DECLARE   (timer_ref2)
 TEST_DECLARE   (fs_event_ref)
+TEST_DECLARE   (fs_poll_ref)
 TEST_DECLARE   (tcp_ref)
 TEST_DECLARE   (tcp_ref2)
 TEST_DECLARE   (tcp_ref3)
@@ -134,6 +135,7 @@ TEST_DECLARE   (spawn_and_ping)
 TEST_DECLARE   (spawn_setuid_fails)
 TEST_DECLARE   (spawn_setgid_fails)
 TEST_DECLARE   (spawn_stdout_to_file)
+TEST_DECLARE   (fs_poll)
 TEST_DECLARE   (kill)
 TEST_DECLARE   (fs_file_noent)
 TEST_DECLARE   (fs_file_nametoolong)
@@ -293,6 +295,7 @@ TASK_LIST_START
 
   TEST_ENTRY  (ref)
   TEST_ENTRY  (idle_ref)
+  TEST_ENTRY  (fs_poll_ref)
   TEST_ENTRY  (async_ref)
   TEST_ENTRY  (prepare_ref)
   TEST_ENTRY  (check_ref)
@@ -360,6 +363,7 @@ TASK_LIST_START
   TEST_ENTRY  (spawn_setuid_fails)
   TEST_ENTRY  (spawn_setgid_fails)
   TEST_ENTRY  (spawn_stdout_to_file)
+  TEST_ENTRY  (fs_poll)
   TEST_ENTRY  (kill)
 #ifdef _WIN32
   TEST_ENTRY  (spawn_detect_pipe_name_collisions_on_windows)
