@@ -49,8 +49,8 @@ def exec_process(cmdline, silent=True, input=None, **kwargs):
 class libuv_build_ext(build_ext):
     libuv_dir      = os.path.join('deps', 'libuv')
     libuv_repo     = 'https://github.com/joyent/libuv.git'
-    libuv_branch   = 'master'
-    libuv_revision = '5c30443'
+    libuv_branch   = 'v0.8'
+    libuv_revision = '5a0f341'
     libuv_patches  = ['patches/c-ares_naptr_support.patch']
 
     user_options = build_ext.user_options
