@@ -59,43 +59,9 @@ format_time(time_t sec, unsigned long nsec)
 
 
 static void
-process_stat(uv_fs_t* req, PyObject **path, PyObject **stat_data, PyObject **errorno) {
-    struct stat *st;
+stat_to_pyobj(uv_statbuf_t *st, PyObject **stat_data) {
     unsigned long ansec, mnsec, cnsec, bnsec;
 
-    ASSERT(req);
-    ASSERT(req->fs_type == UV_FS_STAT || req->fs_type == UV_FS_LSTAT || req->fs_type == UV_FS_FSTAT);
-
-    st = (struct stat *)(req->ptr);
-
-    if (req->path != NULL) {
-        *path = PYUVString_FromString(req->path);
-    } else {
-        *path = Py_None;
-        Py_INCREF(Py_None);
-    }
-
-    if (req->result < 0) {
-        *errorno = PyInt_FromLong((long)req->errorno);
-        *stat_data = Py_None;
-        Py_INCREF(Py_None);
-        return;
-    } else {
-        *errorno = Py_None;
-        Py_INCREF(Py_None);
-    }
-
-    *stat_data = PyStructSequence_New(&StatResultType);
-    if (!stat_data) {
-        PyErr_NoMemory();
-        PyErr_WriteUnraisable(Py_None);
-        Py_DECREF(*path);
-        Py_DECREF(*errorno);
-        *path = NULL;
-        *errorno = NULL;
-        return;
-    }
-
     PyStructSequence_SET_ITEM(*stat_data, 0, PyInt_FromLong((long)st->st_mode));
 #ifdef HAVE_LARGEFILE_SUPPORT
     PyStructSequence_SET_ITEM(*stat_data, 1, PyLong_FromLongLong((PY_LONG_LONG)st->st_ino));
@@ -178,6 +144,47 @@ process_stat(uv_fs_t* req, PyObject **path, PyObject **stat_data, PyObject **err
 }
 
 
+static void
+process_stat(uv_fs_t* req, PyObject **path, PyObject **stat_data, PyObject **errorno) {
+    uv_statbuf_t *st;
+
+    ASSERT(req);
+    ASSERT(req->fs_type == UV_FS_STAT || req->fs_type == UV_FS_LSTAT || req->fs_type == UV_FS_FSTAT);
+
+    st = req->ptr;
+
+    if (req->path != NULL) {
+        *path = PYUVString_FromString(req->path);
+    } else {
+        *path = Py_None;
+        Py_INCREF(Py_None);
+    }
+
+    if (req->result < 0) {
+        *errorno = PyInt_FromLong((long)req->errorno);
+        *stat_data = Py_None;
+        Py_INCREF(Py_None);
+        return;
+    } else {
+        *errorno = Py_None;
+        Py_INCREF(Py_None);
+    }
+
+    *stat_data = PyStructSequence_New(&StatResultType);
+    if (!stat_data) {
+        PyErr_NoMemory();
+        PyErr_WriteUnraisable(Py_None);
+        Py_DECREF(*path);
+        Py_DECREF(*errorno);
+        *path = NULL;
+        *errorno = NULL;
+        return;
+    }
+    stat_to_pyobj(st, stat_data);
+
+}
+
+
 static void
 stat_cb(uv_fs_t* req) {
     PyGILState_STATE gstate = PyGILState_Ensure();
@@ -617,11 +624,7 @@ process_open(uv_fs_t* req, PyObject **path, PyObject **fd, PyObject **errorno)
     } else {
         *errorno = Py_None;
         Py_INCREF(Py_None);
-#ifdef PYUV_WINDOWS
-        *fd = PyInt_FromLong((long)PYUV_FD_TO_WIN32_HANDLE(req->result));
-#else
         *fd = PyInt_FromLong((long)req->result);
-#endif
     }
 }
 
@@ -1180,14 +1183,6 @@ FS_func_fstat(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -1582,14 +1577,6 @@ FS_func_fchmod(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -1919,14 +1906,6 @@ FS_func_fchown(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2065,14 +2044,6 @@ FS_func_close(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2137,14 +2108,6 @@ FS_func_read(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2236,14 +2199,6 @@ FS_func_write(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2334,14 +2289,6 @@ FS_func_fsync(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2405,14 +2352,6 @@ FS_func_fdatasync(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2476,14 +2415,6 @@ FS_func_ftruncate(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2623,19 +2554,6 @@ FS_func_sendfile(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    in_fd = PYUV_WIN32_HANDLE_TO_FD(in_fd);
-    if (in_fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-    out_fd = PYUV_WIN32_HANDLE_TO_FD(out_fd);
-    if (out_fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2776,14 +2694,6 @@ FS_func_futime(PyObject *obj, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-#ifdef PYUV_WINDOWS
-    fd = PYUV_WIN32_HANDLE_TO_FD(fd);
-    if (fd == -1) {
-        PyErr_SetFromWindowsErr(0);
-        return NULL;
-    }
-#endif
-
     fs_req = PyMem_Malloc(sizeof(uv_fs_t));
     if (!fs_req) {
         PyErr_NoMemory();
@@ -2855,6 +2765,7 @@ FS_methods[] = {
 };
 
 
+/* FSEvent handle */
 static PyObject* PyExc_FSEventError;
 
 
@@ -2866,7 +2777,6 @@ on_fsevent_callback(uv_fs_event_t *handle, const char *filename, int events, int
     PyObject *result, *py_filename, *py_events, *errorno;
 
     ASSERT(handle);
-    ASSERT(status == 0);
 
     self = (FSEvent *)handle->data;
     ASSERT(self);
@@ -2890,9 +2800,9 @@ on_fsevent_callback(uv_fs_event_t *handle, const char *filename, int events, int
 
     py_events = PyInt_FromLong((long)events);
 
-    result = PyObject_CallFunctionObjArgs(self->on_fsevent_cb, self, py_filename, py_events, errorno, NULL);
+    result = PyObject_CallFunctionObjArgs(self->callback, self, py_filename, py_events, errorno, NULL);
     if (result == NULL) {
-	PyErr_WriteUnraisable(self->on_fsevent_cb);
+	PyErr_WriteUnraisable(self->callback);
     }
     Py_XDECREF(result);
     Py_DECREF(py_events);
@@ -2946,9 +2856,9 @@ FSEvent_func_start(FSEvent *self, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-    tmp = self->on_fsevent_cb;
+    tmp = self->callback;
     Py_INCREF(callback);
-    self->on_fsevent_cb = callback;
+    self->callback = callback;
     Py_XDECREF(tmp);
 
     Py_RETURN_NONE;
@@ -3007,7 +2917,7 @@ FSEvent_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 static int
 FSEvent_tp_traverse(FSEvent *self, visitproc visit, void *arg)
 {
-    Py_VISIT(self->on_fsevent_cb);
+    Py_VISIT(self->callback);
     HandleType.tp_traverse((PyObject *)self, visit, arg);
     return 0;
 }
@@ -3016,7 +2926,7 @@ FSEvent_tp_traverse(FSEvent *self, visitproc visit, void *arg)
 static int
 FSEvent_tp_clear(FSEvent *self)
 {
-    Py_CLEAR(self->on_fsevent_cb);
+    Py_CLEAR(self->callback);
     HandleType.tp_clear((PyObject *)self);
     return 0;
 }
@@ -3076,6 +2986,253 @@ static PyTypeObject FSEventType = {
     FSEvent_tp_new,                                                 /*tp_new*/
 };
 
+
+/* FSPoll handle */
+static PyObject* PyExc_FSPollError;
+
+
+static void
+on_fspoll_callback(uv_fs_poll_t *handle, int status, const uv_statbuf_t *prev, const uv_statbuf_t *curr)
+{
+    PyGILState_STATE gstate = PyGILState_Ensure();
+    FSPoll *self;
+    PyObject *result, *errorno, *prev_stat_data, *curr_stat_data;
+
+    ASSERT(handle);
+
+    self = (FSPoll *)handle->data;
+    ASSERT(self);
+    /* Object could go out of scope in the callback, increase refcount to avoid it */
+    Py_INCREF(self);
+
+    if (status < 0) {
+        uv_err_t err = uv_last_error(UV_HANDLE_LOOP(self));
+        errorno = PyInt_FromLong((long)err.code);
+        prev_stat_data = Py_None;
+        curr_stat_data = Py_None;
+        Py_INCREF(Py_None);
+        Py_INCREF(Py_None);
+    } else {
+        errorno = Py_None;
+        Py_INCREF(Py_None);
+        prev_stat_data = PyStructSequence_New(&StatResultType);
+        if (!prev_stat_data) {
+            PyErr_NoMemory();
+            PyErr_WriteUnraisable(Py_None);
+            prev_stat_data = Py_None;
+            Py_INCREF(Py_None);
+        } else {
+            stat_to_pyobj((uv_statbuf_t *)prev, &prev_stat_data);
+        }
+        curr_stat_data = PyStructSequence_New(&StatResultType);
+        if (!curr_stat_data) {
+            PyErr_NoMemory();
+            PyErr_WriteUnraisable(Py_None);
+            curr_stat_data = Py_None;
+            Py_INCREF(Py_None);
+        } else {
+            stat_to_pyobj((uv_statbuf_t *)curr, &curr_stat_data);
+        }
+    }
+
+    result = PyObject_CallFunctionObjArgs(self->callback, self, prev_stat_data, curr_stat_data, errorno, NULL);
+    if (result == NULL) {
+        PyErr_WriteUnraisable(self->callback);
+    }
+    Py_XDECREF(result);
+
+    Py_DECREF(self);
+    PyGILState_Release(gstate);
+}
+
+
+static PyObject *
+FSPoll_func_start(FSPoll *self, PyObject *args, PyObject *kwargs)
+{
+    int r;
+    char *path;
+    double interval;
+    PyObject *tmp, *callback;
+
+    static char *kwlist[] = {"path", "callback", "flags", NULL};
+
+    tmp = NULL;
+
+    RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "sOd:start", kwlist, &path, &callback, &interval)) {
+        return NULL;
+    }
+
+    if (!PyCallable_Check(callback)) {
+        PyErr_SetString(PyExc_TypeError, "a callable is required");
+        return NULL;
+    }
+
+    if (interval < 0.0) {
+        PyErr_SetString(PyExc_ValueError, "a positive value or zero is required");
+        return NULL;
+    }
+
+    r = uv_fs_poll_start((uv_fs_poll_t *)UV_HANDLE(self), on_fspoll_callback, path, (unsigned int)interval*1000);
+    if (r != 0) {
+        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_FSPollError);
+        return NULL;
+    }
+
+    tmp = self->callback;
+    Py_INCREF(callback);
+    self->callback = callback;
+    Py_XDECREF(tmp);
+
+    Py_RETURN_NONE;
+}
+
+
+static PyObject *
+FSPoll_func_stop(FSPoll *self)
+{
+    int r;
+
+    RAISE_IF_HANDLE_CLOSED(self, PyExc_HandleClosedError, NULL);
+
+    r = uv_fs_poll_stop((uv_fs_poll_t *)UV_HANDLE(self));
+    if (r != 0) {
+        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_FSPollError);
+        return NULL;
+    }
+
+    Py_XDECREF(self->callback);
+    self->callback = NULL;
+
+    Py_RETURN_NONE;
+}
+
+
+static int
+FSPoll_tp_init(FSPoll *self, PyObject *args, PyObject *kwargs)
+{
+    int r;
+    uv_fs_poll_t *uv_fspoll = NULL;
+    Loop *loop;
+    PyObject *tmp = NULL;
+
+    UNUSED_ARG(kwargs);
+
+    if (UV_HANDLE(self)) {
+        PyErr_SetString(PyExc_FSPollError, "Object already initialized");
+        return -1;
+    }
+
+    if (!PyArg_ParseTuple(args, "O!:__init__", &LoopType, &loop)) {
+        return -1;
+    }
+
+    tmp = (PyObject *)((Handle *)self)->loop;
+    Py_INCREF(loop);
+    ((Handle *)self)->loop = loop;
+    Py_XDECREF(tmp);
+
+    uv_fspoll = PyMem_Malloc(sizeof(uv_fs_poll_t));
+    if (!uv_fspoll) {
+        PyErr_NoMemory();
+        Py_DECREF(loop);
+        return -1;
+    }
+
+    r = uv_fs_poll_init(UV_HANDLE_LOOP(self), uv_fspoll);
+    if (r != 0) {
+        RAISE_UV_EXCEPTION(UV_HANDLE_LOOP(self), PyExc_FSPollError);
+        Py_DECREF(loop);
+        return -1;
+    }
+    uv_fspoll->data = (void *)self;
+    UV_HANDLE(self) = (uv_handle_t *)uv_fspoll;
+
+    return 0;
+}
+
+
+static PyObject *
+FSPoll_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
+{
+    FSPoll *self = (FSPoll *)HandleType.tp_new(type, args, kwargs);
+    if (!self) {
+        return NULL;
+    }
+    return (PyObject *)self;
+}
+
+
+static int
+FSPoll_tp_traverse(FSPoll *self, visitproc visit, void *arg)
+{
+    Py_VISIT(self->callback);
+    HandleType.tp_traverse((PyObject *)self, visit, arg);
+    return 0;
+}
+
+
+static int
+FSPoll_tp_clear(FSPoll *self)
+{
+    Py_CLEAR(self->callback);
+    HandleType.tp_clear((PyObject *)self);
+    return 0;
+}
+
+
+static PyMethodDef
+FSPoll_tp_methods[] = {
+    { "start", (PyCFunction)FSPoll_func_start, METH_VARARGS, "Start the FSPoll handle." },
+    { "stop", (PyCFunction)FSPoll_func_stop, METH_NOARGS, "Stop the FSPoll handle." },
+    { NULL }
+};
+
+
+static PyTypeObject FSPollType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "pyuv.fs.FSPoll",                                               /*tp_name*/
+    sizeof(FSPoll),                                                 /*tp_basicsize*/
+    0,                                                              /*tp_itemsize*/
+    0,                                                              /*tp_dealloc*/
+    0,                                                              /*tp_print*/
+    0,                                                              /*tp_getattr*/
+    0,                                                              /*tp_setattr*/
+    0,                                                              /*tp_compare*/
+    0,                                                              /*tp_repr*/
+    0,                                                              /*tp_as_number*/
+    0,                                                              /*tp_as_sequence*/
+    0,                                                              /*tp_as_mapping*/
+    0,                                                              /*tp_hash */
+    0,                                                              /*tp_call*/
+    0,                                                              /*tp_str*/
+    0,                                                              /*tp_getattro*/
+    0,                                                              /*tp_setattro*/
+    0,                                                              /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,                        /*tp_flags*/
+    0,                                                              /*tp_doc*/
+    (traverseproc)FSPoll_tp_traverse,                               /*tp_traverse*/
+    (inquiry)FSPoll_tp_clear,                                       /*tp_clear*/
+    0,                                                              /*tp_richcompare*/
+    0,                                                              /*tp_weaklistoffset*/
+    0,                                                              /*tp_iter*/
+    0,                                                              /*tp_iternext*/
+    FSPoll_tp_methods,                                              /*tp_methods*/
+    0,                                                              /*tp_members*/
+    0,                                                              /*tp_getsets*/
+    0,                                                              /*tp_base*/
+    0,                                                              /*tp_dict*/
+    0,                                                              /*tp_descr_get*/
+    0,                                                              /*tp_descr_set*/
+    0,                                                              /*tp_dictoffset*/
+    (initproc)FSPoll_tp_init,                                       /*tp_init*/
+    0,                                                              /*tp_alloc*/
+    FSPoll_tp_new,                                                  /*tp_new*/
+};
+
+
+
 #ifdef PYUV_PYTHON3
 static PyModuleDef pyuv_fs_module = {
     PyModuleDef_HEAD_INIT,
@@ -3107,7 +3264,10 @@ init_fs(void)
     PyModule_AddIntMacro(module, UV_FS_SYMLINK_JUNCTION);
 
     FSEventType.tp_base = &HandleType;
+    FSPollType.tp_base = &HandleType;
+
     PyUVModule_AddType(module, "FSEvent", &FSEventType);
+    PyUVModule_AddType(module, "FSPoll", &FSPollType);
 
     return module;
 }
