@@ -226,17 +226,20 @@ static PyTypeObject ProcessType;
 
 /* FSEvent */
 typedef struct {
-    PyObject_HEAD
-    PyObject *weakreflist;
-    Loop *loop;
-    PyObject *on_fsevent_cb;
-    PyObject *on_close_cb;
-    PyObject *data;
-    uv_fs_event_t *uv_handle;
+    Handle handle;
+    PyObject *callback;
 } FSEvent;
 
 static PyTypeObject FSEventType;
 
+/* FSPoll */
+typedef struct {
+    Handle handle;
+    PyObject *callback;
+} FSPoll;
+
+static PyTypeObject FSPollType;
+
 /* DNSResolver */
 typedef struct {
     PyObject_HEAD
@@ -301,13 +304,7 @@ PyUVModule_AddObject(PyObject *module, const char *name, PyObject *value)
 
 /* borrowed from pyev */
 #ifdef PYUV_WINDOWS
-
-#define PYUV_MAXSTDIO 2048
-
-/* Convert a Windows handle into a usable file descriptor */
-#define PYUV_WIN32_HANDLE_TO_FD(handle) _open_osfhandle (handle, 0)
-#define PYUV_FD_TO_WIN32_HANDLE(fd) _get_osfhandle (fd)
-
+    #define PYUV_MAXSTDIO 2048
 #endif
 
 #endif
