@@ -21,7 +21,7 @@ def on_ipc_connection(handle, error):
     conn = pyuv.TCP(loop)
     tcp_server.accept(conn)
     conn.close()
-    channel.write("accepted_connection", on_channel_write)
+    channel.write(b"accepted_connection", on_channel_write)
     connection_accepted = True
 
 
@@ -38,7 +38,7 @@ tcp_server.bind(("0.0.0.0", TEST_PORT))
 if not listen_after_write:
     tcp_server.listen(on_ipc_connection, 12)
 
-channel.write2("hello", tcp_server)
+channel.write2(b"hello", tcp_server)
 
 if listen_after_write:
     tcp_server.listen(on_ipc_connection, 12)
