@@ -3,7 +3,7 @@ import os
 import shutil
 import stat
 
-from common import platform_skip, unittest2
+from common import unittest2
 import pyuv
 
 
@@ -55,14 +55,13 @@ class FSTestStat(unittest2.TestCase):
         self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)
 
 
-@platform_skip(["win32"])
 class FSTestLstat(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
         with open(TEST_FILE, 'w') as f:
             f.write('test')
-        os.symlink(TEST_FILE, TEST_LINK)
+        pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0)
 
     def tearDown(self):
         os.remove(TEST_FILE)
@@ -82,11 +81,10 @@ class FSTestFstat(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
-        self.file = open(TEST_FILE, 'w')
-        self.file.write('test')
+        with open(TEST_FILE, 'w') as f:
+            f.write('test')
 
     def tearDown(self):
-        self.file.close()
         os.remove(TEST_FILE)
 
     def fstat_cb(self, loop, path, stat_data, errorno):
@@ -94,12 +92,16 @@ class FSTestFstat(unittest2.TestCase):
 
     def test_fstat(self):
         self.errorno = None
-        pyuv.fs.fstat(self.loop, self.file.fileno(), self.fstat_cb)
+        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
+        pyuv.fs.fstat(self.loop, fd, self.fstat_cb)
         self.loop.run()
+        pyuv.fs.close(self.loop, fd)
         self.assertEqual(self.errorno, None)
 
     def test_fstat_sync(self):
-        self.stat_data = pyuv.fs.fstat(self.loop, self.file.fileno())
+        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
+        self.stat_data = pyuv.fs.fstat(self.loop, fd)
+        pyuv.fs.close(self.loop, fd)
         self.assertNotEqual(self.stat_data, None)
 
 
@@ -350,7 +352,6 @@ class FSTestLink(unittest2.TestCase):
         self.assertEqual(os.stat(TEST_FILE).st_ino, os.stat(TEST_LINK).st_ino)
 
 
-@platform_skip(["win32"])
 class FSTestSymlink(unittest2.TestCase):
 
     def setUp(self):
@@ -380,14 +381,13 @@ class FSTestSymlink(unittest2.TestCase):
         self.assertTrue(os.stat(TEST_LINK).st_mode & stat.S_IFLNK)
 
 
-@platform_skip(["win32"])
 class FSTestReadlink(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
         with open(TEST_FILE, 'w') as f:
             f.write('test')
-        os.symlink(TEST_FILE, TEST_LINK)
+        pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0)
 
     def tearDown(self):
         os.remove(TEST_FILE)
@@ -437,11 +437,10 @@ class FSTestFchown(unittest2.TestCase):
 
     def setUp(self):
         self.loop = pyuv.Loop.default_loop()
-        self.file = open(TEST_FILE, 'w')
-        self.file.write('test')
+        with open(TEST_FILE, 'w') as f:
+            f.write('test')
 
     def tearDown(self):
-        self.file.close()
         os.remove(TEST_FILE)
 
     def fchown_cb(self, loop, path, errorno):
@@ -449,12 +448,16 @@ class FSTestFchown(unittest2.TestCase):
 
     def test_fchown(self):
         self.errorno = None
-        pyuv.fs.fchown(self.loop, self.file.fileno(), -1, -1, self.fchown_cb)
+        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD | stat.S_IWRITE)
+        pyuv.fs.fchown(self.loop, fd, -1, -1, self.fchown_cb)
         self.loop.run()
+        pyuv.fs.close(self.loop, fd)
         self.assertEqual(self.errorno, None)
 
     def test_fchown_sync(self):
-        pyuv.fs.fchown(self.loop, self.file.fileno(), -1, -1)
+        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD | stat.S_IWRITE)
+        pyuv.fs.fchown(self.loop, fd, -1, -1)
+        pyuv.fs.close(self.loop, fd)
 
 
 class FSTestOpen(unittest2.TestCase):
@@ -899,6 +902,54 @@ class FSEventTest(unittest2.TestCase):
         self.assertTrue(self.events & pyuv.fs.UV_CHANGE)
 
 
+class FSPollTest(unittest2.TestCase):
+
+    def setUp(self):
+        self.loop = pyuv.Loop.default_loop()
+
+    def tearDown(self):
+        try:
+            os.remove(TEST_FILE)
+        except OSError:
+            pass
+
+    def _touch_file(self):
+        with open(TEST_FILE, 'w+') as f:
+            self.count += 1
+            for i in range(self.count+1):
+                f.write('*')
+
+    def on_timer(self, timer):
+        self._touch_file()
+
+    def on_fspoll(self, handle, prev_stat, curr_stat, error):
+        if self.poll_cb_called == 0:
+            self.assertEqual(error, pyuv.errno.UV_ENOENT)
+            self._touch_file()
+        elif self.poll_cb_called == 1:
+            self.timer.start(self.on_timer, 0.02, 0.0)
+        elif self.poll_cb_called == 2:
+            self.timer.start(self.on_timer, 0.2, 0.0)
+        elif self.poll_cb_called == 3:
+            os.remove(TEST_FILE)
+        elif self.poll_cb_called == 4:
+            self.assertEqual(error, pyuv.errno.UV_ENOENT)
+            self.fs_poll.close()
+            self.timer.close()
+        else:
+            self.fail('This should not happen')
+        self.poll_cb_called += 1
+
+    def test_fspoll1(self):
+        self.count = 0
+        self.poll_cb_called = 0
+        self.timer = pyuv.Timer(self.loop)
+        self.fs_poll = pyuv.fs.FSPoll(self.loop)
+        self.fs_poll.start(TEST_FILE, self.on_fspoll, 0.1)
+        self.loop.run()
+        self.assertEqual(self.poll_cb_called, 5)
+
+
 if __name__ == '__main__':
     unittest2.main(verbosity=2)
 
